package com.example.nto.controller;

import com.example.nto.entity.Employee;
import com.example.nto.service.EmployeeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.annotation.Secured;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

@Controller
@RequestMapping("/api/admin/")
public class AdminController {

    @Autowired
    private EmployeeService employeeService;


    public static class DeleteEmployeeRequest {
        private Long id;


        public Long getId() {
            return id;
        }

        public void setId(Long id) {
            this.id = id;
        }
    }

    public static class UpdateEmployeeRequest {
        private Long id;
        private String field;

        public Long getId() {
            return id;
        }

        public void setId(Long id) {
            this.id = id;
        }

        public String getField() {
            return field;
        }

        public void setField(String field) {
            this.field = field;
        }
    }

    /**
     * Такой же метод, как и getEmployeeInfo, только для админов по логину
     * @return
     */
    @PreAuthorize("hasAnyRole('ADMIN')")
    @GetMapping("/{value}/info")
    public Employee getEmployeeInfoAdmin(@PathVariable String value) {
        return employeeService.getEmployeeInfoById(Long.parseLong(value));
    }

    @PreAuthorize("hasAnyRole('ADMIN')")
    @DeleteMapping("/employee/delete/")
    public ResponseEntity<String> deleteEmployee(@RequestBody DeleteEmployeeRequest deleteEmployeeRequest) {
        employeeService.deleteEmployee(deleteEmployeeRequest.id);
        return ResponseEntity.status(HttpStatus.OK).build();
    }

    @PreAuthorize("hasRole('DOLBOEB')")
    @PutMapping("/employee/update/login/")
    public ResponseEntity<String> updateEmployeeLogin(@RequestBody UpdateEmployeeRequest updateEmployeeRequest) {

        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();

        employeeService.updateEmployeeLogin(updateEmployeeRequest.id, updateEmployeeRequest.field);
        return ResponseEntity.status(HttpStatus.OK).build();
    }

    @PreAuthorize("hasAnyRole('ADMIN')")
    @PutMapping("/employee/update/avatar/")
    public ResponseEntity<String> updateEmployeeAvatar(@RequestBody UpdateEmployeeRequest updateEmployeeRequest) {
        employeeService.updateEmployeeAvatar(updateEmployeeRequest.id, updateEmployeeRequest.field);
        return ResponseEntity.status(HttpStatus.OK).build();
    }

    @PreAuthorize("hasAnyRole('ADMIN')")
    @PutMapping("/employee/update/position/")
    public ResponseEntity<String> updatePosition(@RequestBody UpdateEmployeeRequest updateEmployeeRequest) {
        employeeService.updateEmployeePosition(updateEmployeeRequest.id, updateEmployeeRequest.field);
        return ResponseEntity.status(HttpStatus.OK).build();
    }
}
