package com.example.nto.controller;

import com.example.nto.entity.Employee;
import com.example.nto.repository.CodeRepository;
import com.example.nto.service.EmployeeService;
import org.springframework.boot.autoconfigure.ldap.embedded.EmbeddedLdapProperties;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.annotation.Secured;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Optional;

@RestController
public class EmployeeController {
    private final EmployeeService employeeService;
    private final CodeRepository codeRepository;

    public EmployeeController(EmployeeService employeeService, CodeRepository codeRepository) {
        this.employeeService = employeeService;
        this.codeRepository = codeRepository;
    }

    /**
     * Эндпоинт для проверки присутствия логина в емплоеееее таблице
     *
     * @return статус код, смотри документацию к тз
     */
    @GetMapping("/api/auth")
    public ResponseEntity<String> authAttempt(HttpServletRequest request) {

        String login = SecurityContextHolder.getContext().getAuthentication().getName();

        try {
            if (employeeService.checkEmployeeExists(login)) {
                return ResponseEntity.status(HttpStatus.ACCEPTED).build(); // Логин найден ДВЕСТИ
            }
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).build(); // Логин не найден 401 неавторизован

        } catch (
                Exception e) // Я понял статус "что-то пошло не так", как то, что произошла какая-то ошибка. Вообще по идее должен быть статус 500, но допустим
        {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).build();
        }
    }

    /**
     * Получить информацию по емплоеееее
     *
     * @return емплоеееееее иначе 401 бан бан бан бан, а если ошибка, то 400 БАААААН
     */
    @GetMapping("/api/info")
    public ResponseEntity<Employee> getEmployeeInfo() {
        String login = SecurityContextHolder.getContext().getAuthentication().getName();

        try {
            Optional<Employee> employee = employeeService.getEmployeeInfo(login);
            return employee.map(ResponseEntity::ok).orElseGet(() -> ResponseEntity.status(HttpStatus.UNAUTHORIZED).build());
        } catch (
                Exception e) // Я понял статус "что-то пошло не так", как то, что произошла какая-то ошибка. Вообще по идее должен быть статус 500, но допустим
        {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).build();
        }
    }

    /**
     * Проверка на дверь открыть не открыть
     * @param value код двери
     * @return статус код отвта
     */
    @PatchMapping("api/{value}/open")
    public ResponseEntity<String> openDoorAttempt(@RequestBody Long value) {
        try {
            String login = SecurityContextHolder.getContext().getAuthentication().getName();

            if (!employeeService.checkEmployeeExists(login)) {
                return ResponseEntity.status(HttpStatus.UNAUTHORIZED).build();
            }

            if (!codeRepository.existsByValue(value)) {
                return ResponseEntity.status(HttpStatus.BAD_REQUEST).build();
            }

            employeeService.updateLastVisit(login);
            return ResponseEntity.status(HttpStatus.ACCEPTED).build();

        } catch (Exception e) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).build();
        }

    }


}
